﻿using System;
using System.Collections.ObjectModel;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using System.Linq;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using System.Windows.Threading;
using System.Diagnostics;
using System.Windows.Controls;

namespace DisplayASimpleMapForWPF
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private DispatcherTimer timer;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void WpfMap_Loaded(object sender, RoutedEventArgs e)
        {
            wpfMap1.MapUnit = GeographyUnit.DecimalDegree;
            WorldMapKitWmsWpfOverlay worldMapKitOverlay = new WorldMapKitWmsWpfOverlay();
            wpfMap1.Overlays.Add(worldMapKitOverlay);

            ShapeFileFeatureLayer worldLayer = new ShapeFileFeatureLayer(@"..\..\Data\Countries02.shp");
            worldLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            worldLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.FromArgb(100, GeoColor.SimpleColors.Green));

            LayerOverlay staticOverlay = new LayerOverlay();
            staticOverlay.Layers.Add("WorldLayer", worldLayer);
            staticOverlay.TileType = TileType.SingleTile;
            wpfMap1.Overlays.Add(staticOverlay);

            wpfMap1.CurrentExtent = new RectangleShape(-133.2515625, 89.2484375, 126.9046875, -88.290625);

            foreach (Overlay overlay in wpfMap1.Overlays.OfType<TileOverlay>()
                                .Where((to => to.TileType != TileType.SingleTile && to.IsVisible)))
            {
                TileOverlay tileOverlay = overlay as TileOverlay;
                tileOverlay.DrawTilesProgressChanged += new System.EventHandler<DrawTilesProgressChangedTileOverlayEventArgs>(tileOverlay_DrawTilesProgressChanged);
            }

            wpfMap1.Refresh();
        }

        void tileOverlay_DrawTilesProgressChanged(object sender, DrawTilesProgressChangedTileOverlayEventArgs e)
        {
            var tileOverlays = wpfMap1.Overlays
                                .OfType<TileOverlay>()
                                .Where((to => to.TileType != TileType.SingleTile && to.IsVisible));

            bool isFinished = tileOverlays.All(o => CheckIsDrawn(o));

            if (timer == null)
            {
                timer = new DispatcherTimer();
                timer.Interval = TimeSpan.FromSeconds(1);
                timer.Tick += (s, a) =>
                {
                    timer.Stop();

                    string id = Guid.NewGuid().ToString();
                    System.Drawing.Bitmap map = wpfMap1.GetBitmap(1000, 800);
                    map.Save("d:\\" + id + ".png");
                    Process.Start("d:\\" + id + ".png");
                };
            }

            if (isFinished)
            {
                timer.Stop();
                timer.Start();
            }
        }

        private static bool CheckIsDrawn(TileOverlay tileOverlay)
        {
            bool isFinished = true;
            Canvas drawingCanvas = ((Canvas)tileOverlay.OverlayCanvas.Children[0]);
            if (drawingCanvas.Children.Count != 0)
            {
                foreach (ThinkGeo.MapSuite.WpfDesktopEdition.Tile tile in drawingCanvas.Children)
                {
                    if (!tile.IsOpened)
                    {
                        isFinished = false;
                        break;
                    }
                }
            }
            else isFinished = false;

            return isFinished;
        }
    }
}
